#!/usr/bin/perl -w

use strict;
use warnings;
use DBI;
use RRDs;
use FindBin qw($Bin);

our %config;
require "$Bin/../lib/graylib.pl";

my %actions;

# called every minute
$actions{permit} = sub {
  my ($dbh) = @_;
  my $sql = "UPDATE $config{db_tbl_gray} SET $config{db_field_ok} = '1', ";
  $sql .= "$config{db_field_ts_ok} = now() WHERE ";
  $sql .= "$config{db_field_ok} = '0' AND ";
  $sql .= "now() - $config{delay_permit} >= $config{db_field_ts_entry}";
  $dbh->do($sql);
};

# called every 5 minutes
$actions{graph} = sub {
  my ($dbh) = @_;

  if (!-e $config{rrdfile}) {
    # hmmm, step implies crontab entry here...
    RRDs::create($config{rrdfile}, "--start", "1000000000", "--step", 300,
                 "DS:deny:GAUGE:600:U:U", "DS:halfopen:GAUGE:600:0:U",
                 "DS:permit:GAUGE:600:U:U", "RRA:MAX:0:1:210240");
    my $err = RRDs::error;
    die "create $config{rrdfile} failed: $err" if $err;
  }
  # get numbers
  my $cnt_deny = &dbcount("'0'");
  my $cnt_halfopen = &dbcount("'1' AND " .
    "$config{db_field_ts_entry} = $config{db_field_ts_latest}");
  my $cnt_permit = &dbcount("'1' AND " .
    "$config{db_field_ts_entry} != $config{db_field_ts_latest}");
  if (defined $cnt_deny && defined $cnt_halfopen && defined $cnt_permit) {
    # update rrd file
    RRDs::update($config{rrdfile}, "-t", "deny:halfopen:permit",
                 "N:$cnt_deny:$cnt_halfopen:$cnt_permit");
    my $err = RRDs::error;
    die "update $config{rrdfile} failed: $err" if $err;
  } else {
    $cnt_deny = $cnt_halfopen = $cnt_permit = "?";
  }
  # make pictures
  foreach my $period("1h", "1d", "1w", "1m", "1y") {
    my $fn = $config{picpath} . "/graylist" . $period . ".png";
    RRDs::graph($fn, "-s", "end-" . $period, "-l", "0", "-w", "500",
                "-t", "Graylisting per " . $period,
                "DEF:deny=$config{rrdfile}:deny:MAX",
                "DEF:halfopen=$config{rrdfile}:halfopen:MAX",
                "DEF:permit=$config{rrdfile}:permit:MAX",
                "AREA:permit#00FF00:permit\\: $cnt_permit\\n",
                "STACK:halfopen#0000FF:half open\\: $cnt_halfopen\\n",
                "STACK:deny#FF0000:deny\\: $cnt_deny");
    my $err = RRDs::error;
    die "graph $fn ($config{rrdfile}) failed: $err" if $err;
  }

  if (!-e $config{rrdfile2}) {
    # hmmm, step implies crontab entry here...
    RRDs::create($config{rrdfile2}, "--start", "1000000000", "--step", 300,
                 "DS:mails:COUNTER:600:U:U", "DS:recipients:COUNTER:600:0:U",
                 "RRA:MIN:0:1:210240", "RRA:AVERAGE:0:1:210240",
                 "RRA:MAX:0:1:210240", "RRA:LAST:0:1:210240");
    my $err = RRDs::error;
    die "create $config{rrdfile2} failed: $err" if $err;
  }
  # get numbers
  my $sql = "SELECT COUNT(*), SUM($config{db_field_count}) ";
  $sql .= "FROM $config{db_tbl_mailcount}";
  my ($cnt_mails, $cnt_recipients) = $dbh->selectrow_array($sql);
  if (defined $cnt_mails && defined $cnt_recipients) {
    # update rrd file
    RRDs::update($config{rrdfile2}, "-t", "mails:recipients",
                 "N:$cnt_mails:$cnt_recipients");
    my $err = RRDs::error;
    die "update $config{rrdfile2} failed: $err" if $err;
  }
  # make pictures
  foreach my $period("1h", "1d", "1w", "1m", "1y") {
    my $fn = $config{picpath} . "/mailcount" . $period . ".png";
    RRDs::graph($fn, "-s", "end-" . $period, "-l", "0", "-w", "500",
                "-t", "Mails per " . $period,
                "DEF:minmails=$config{rrdfile2}:mails:MIN",
                "DEF:avgmails=$config{rrdfile2}:mails:AVERAGE",
                "DEF:maxmails=$config{rrdfile2}:mails:MAX",
                "DEF:lstmails=$config{rrdfile2}:mails:LAST",
                "DEF:minrcpts=$config{rrdfile2}:recipients:MIN",
                "DEF:avgrcpts=$config{rrdfile2}:recipients:AVERAGE",
                "DEF:maxrcpts=$config{rrdfile2}:recipients:MAX",
                "DEF:lstrcpts=$config{rrdfile2}:recipients:LAST",
                "CDEF:varmails=maxmails,minmails,-",
                "CDEF:varrcpts=maxrcpts,minrcpts,-",
                "AREA:minmails",
                "STACK:varmails#bbbbbb",
                "LINE1:avgmails#00ff00:Mails     ",
                "GPRINT:minmails:MIN:Min\\:%6.1lf %s",
                "GPRINT:avgmails:AVERAGE:Avg\\:%6.1lf %s",
                "GPRINT:maxmails:MAX:Max\\:%6.1lf %s",
                "GPRINT:lstmails:LAST:Cur\\:%6.1lf %s\\n",
                "AREA:minrcpts",
                "STACK:varrcpts#ccccff",
                "LINE1:avgrcpts#0000ff:Recipients",
                "GPRINT:minrcpts:MIN:Min\\:%6.1lf %s",
                "GPRINT:avgrcpts:AVERAGE:Avg\\:%6.1lf %s",
                "GPRINT:maxrcpts:MAX:Max\\:%6.1lf %s",
                "GPRINT:lstrcpts:LAST:Cur\\:%6.1lf %s\\n");
    my $err = RRDs::error;
    die "graph $fn ($config{rrdfile2}) failed: $err" if $err;
  }
};

# called once an hour
$actions{clean} = sub {
  my ($dbh) = @_;
  my $sql = "DELETE FROM $config{db_tbl_gray} WHERE $config{db_field_ok} = '1' ";
  $sql .= "AND $config{db_field_ts_entry} = $config{db_field_ts_latest} AND ";
  $sql .= "now() - $config{delay_halfopen} >= $config{db_field_ts_entry}";
  $dbh->do($sql);
};

# called once a day
$actions{scrub} = sub {
  my ($dbh) = @_;
  my $sql = "DELETE FROM $config{db_tbl_gray} WHERE $config{db_field_ok} = '1' ";
  $sql .= "AND now() - $config{delay_max} >= $config{db_field_ts_latest}";
  $dbh->do($sql);
};

# get action from commandline
my $action = shift;
if (!$action || !exists $actions{$action}) {
  die "usage: $0 <permit | graph | clean | scrub>";
}

# load config
my $err = &load_config();
warn $err if $err;

# avoid multiple instances
my $lockfile = "$config{lockfile}.$action";
open(FH, ">$lockfile") || die "$lockfile: $!";
flock FH, 6 || exit 0;

# open database and do something
my $dbh = &dbopen(undef);
&{$actions{$action}}($dbh);
$dbh->disconnect();

# byebye
flock FH, 8 || die "$lockfile: $!";
close(FH);
unlink $lockfile;

sub dbcount ()
{
  my $sql = "SELECT COUNT(*) FROM $config{db_tbl_gray} ";
  $sql .= "WHERE $config{db_field_ok} = $_[0]";
  my @row = $dbh->selectrow_array($sql);
  return (@row ? $row[0] : undef);
}
